
function clik = dataparam_indeplike(theta, ss, dtype, dparam)
%dataparam_indeplike     Likelihoods of several data points, given fixed parameters

numdata = size(ss,2);

switch dtype,
case 1,   % Discrete multinomial
  clik = log(theta(ss',:));

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  numcc = size(theta,2);
  mu    = theta(1:dim,:);
  sigma = reshape(theta(dim+1:end,:), dim, dim, numcc);

  clik = zeros(numdata,numcc);
  for cc = 1:numcc
    sigmaChol  = chol(sigma(:,:,cc));
    delmu      = sigmaChol' \ (ss - mu(:,cc*ones(1,numdata)));
    clik(:,cc) = -sum(log(diag(sigmaChol))) - 0.5*sum(delmu.^2,1)';
  end
  clik = clik - 0.5*dim*log(2*pi);

case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  numcc = size(theta,2);
  mu    = theta(1:dim,:);
  sigma = sqrt(theta(dim+1:end,:));

  clik = zeros(numdata,numcc);
  for cc = 1:numcc
    delmu      = (ss - mu(:,cc*ones(1,numdata))) ./ sigma(:,cc*ones(1,numdata));
    clik(:,cc) = -sum(log(sigma(:,cc))) - 0.5*sum(delmu.^2,1)';
  end
  clik = clik - 0.5*dim*log(2*pi);

case 4,   % Inverse-Wishart (zero mean)
  dim   = dparam(1);
  numcc = size(theta,2);
  sigma = reshape(theta, dim, dim, numcc);

  clik = zeros(numdata,numcc);
  for cc = 1:numcc
    sigmaChol  = chol(sigma(:,:,cc));
    delmu      = sigmaChol' \ ss;
    clik(:,cc) = -sum(log(diag(sigmaChol))) - 0.5*sum(delmu.^2,1)';
  end
  clik = clik - 0.5*dim*log(2*pi);

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim   = dparam(1);
  numcc = size(theta,2);
  sigma = sqrt(theta);

  clik = zeros(numdata,numcc);
  for cc = 1:numcc
    delmu      = ss ./ sigma(:,cc*ones(1,numdata));
    clik(:,cc) = -sum(log(sigma(:,cc))) - 0.5*sum(delmu.^2,1)';
  end
  clik = clik - 0.5*dim*log(2*pi);

end

