
function data = dataparam_sampdata(theta, numsamp, dtype, dparam)
%dataparam_sampdata     Sample data items using given parameters

switch dtype,
case 1,   % Discrete multinomial
  data = multinomial_rnd(theta, numsamp)';

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  mu    = theta(1:dim);
  sigma = reshape(theta(dim+1:end),dim,dim);

  data  = repmat(mu,1,numsamp) + chol(sigma)'*randn(dim,numsamp);

case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  mu    = theta(1:dim);
  sigma = theta(dim+1:end);

  data  = repmat(mu,1,numsamp) + ...
          repmat(sqrt(sigma),1,numsamp) .* randn(dim,numsamp);

case 4,   % Inverse-Wishart (zero mean)
  dim   = dparam(1);
  sigma = reshape(theta, dim, dim);

  data  = chol(sigma)'*randn(dim,numsamp);

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim   = dparam(1);
  sigma = theta;

  data  = repmat(sqrt(sigma),1,numsamp) .* randn(dim,numsamp);

end

