
function rr = trandata_samp(qq, nd, pp, nt, ss, dtype, dparam, ttype, tparam)
%trandata_samp    Sample single transformation from posterior given data

switch ttype,
case 1,   % Inverse-Wishart (constrained zero mean, assumes dtype=2)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numobs = size(ss,2); 
  sumobs = sum(ss,2);

  % cached statistics of transformations and data
  tranchol = reshape(pp(1:end-1), dim, dim);
  datasum  = qq(1:dim);
  datamean = datasum ./ kappa;
  datachol = reshape(qq(dim+1:end-1), dim, dim);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa*(dataNu-dim-1))/(kappa+1);

  % compute posterior inverse covariance
  trancholinv = tranchol \ eye(dim);
  traninv     = trannorm*trancholinv*trancholinv';
  datacholinv = datachol \ eye(dim);
  datainv     = datanorm*datacholinv*datacholinv';
  postinv     = traninv + numobs*datainv;
  postinvchol = chol(postinv);

  % compute posterior mean
  postmean = postinv \ (datainv * (sumobs-numobs*datamean));

  % single sample from posterior distribution
  rr = postmean + postinvchol \ randn(dim,1);

case 2,   % Diagonal Inverse-Wishart (constrained zero mean, assumes dtype=3)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numobs = size(ss,2); 
  sumobs = sum(ss,2);

  % cached statistics of transformations and data
  tranchol = pp(1:dim);
  datamean = qq(1:dim) / kappa;
  datachol = qq(dim+1:end-1);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa*(dataNu-dim-1))/(kappa+1);

  % compute posterior inverse covariance
  traninv = trannorm ./ (tranchol.^2);
  datainv = datanorm ./ (datachol.^2);
  postinv = traninv + numobs*datainv;

  % compute posterior mean
  postmean = (datainv .* (sumobs-numobs*datamean)) ./ postinv;

  % single sample from posterior distribution
  rr = postmean + randn(dim,1) ./ sqrt(postinv);

end

