
function clik = transtat_condlike(pp, nd, rr, ttype, tparam)
%transtat_condlike     Evaluate conditional likelihoods for multiple classes

numcc = size(pp,2);

switch ttype,
case 0,   % No transformations
  clik  = zeros(1,numcc);

case 1,   % Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  tranchol  = reshape(pp(1:end-1,:), dim, dim, numcc);
  clik      = pp(end,:);
  innernorm = 0.5*(nu-dim-1);

  for cc = 1:numcc
    delmu    = tranchol(:,:,cc)' \ rr;
    clik(cc) = clik(cc) - innernorm(cc)*sum(delmu.^2);
  end

case 2,   % Diagonal Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  tranchol  = pp(1:end-1,:);
  innernorm = 0.5*(nu-dim-1);
  delmu     = rr(:,ones(1,numcc)) ./ tranchol;
  clik      = pp(end,:) - innernorm.*sum(delmu.^2);

end

