
function theta = transtat_estparam(pp, nd, ttype, tparam)
%transtat_estparam     Estimate most likely parameters for multiple classes

switch ttype,
case 0,   % No transformations
  theta = zeros(1,length(nd));

case 1,   % Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  numcc = size(pp,2);
  theta = zeros(dim^2,numcc);

  tranchol = reshape(pp(1:end-1,:), dim, dim, numcc);
  covnorm  = 1 ./ (nu-dim-1);
  
  for cc = 1:numcc
    sigma = covnorm(cc)*tranchol(:,:,cc)'*tranchol(:,:,cc);
    theta(:,cc) = sigma(:);
  end

case 2,   % Diagonal Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  numcc = size(pp,2);

  if numcc > 0
    tranchol = pp(1:dim,:);
    covnorm  = 1 ./ (nu-dim-1);

    theta = (tranchol.^2) .* covnorm(ones(dim,1),:);
  else
    theta = zeros(dim,0);
  end

end

