
function objpart = clearlabels(objpart, groups, clearcat)
%clearlabels     Clear assignments of data to parts for specified groups
%
%    OP = clearlabels(OP, groups, clearcat)
%
%  Clears internal assignments of data to classes for specified groups
%  (by default, all).  Useful to test learning from synthetic data.
%  If clearcat flag set, also clears object category labels (default=false).

  % Default parameters
  if nargin < 2
    groups = [];
  end
  if nargin < 3
    clearcat = 0;
  end

  if isempty(groups)
    groups = [1:objpart.numgroup];
  end

  % Hyperparameters
  dtype    = objpart.dtype;
  dparam   = objpart.dparam;
  numtype  = objpart.numtype;

  % Class (part) state variables
  classqq = objpart.classqq;
  classnd = objpart.classnd;
  objnd   = objpart.objnd;

  % Clear labels for each specified group
  for j = 1:length(groups)
    jj = groups(j);

    % State variables for current group
    numdata  = objpart.numdata(jj);
    datass   = objpart.datass{jj};
    datacc   = objpart.datacc{jj};
    category = objpart.category(jj);

    % Remove each data item from cached statistics
    for ii = find(datacc > 0)
      for dd = 1:numtype
        classqq{dd}(:,datacc(ii)) = datastat_del(classqq{dd}(:,datacc(ii)), ...
          classnd(datacc(ii)), datass{dd}(:,ii), dtype(dd), dparam{dd});
      end

      classnd(datacc(ii)) = classnd(datacc(ii)) - 1;
      objnd(category,datacc(ii)) = objnd(category,datacc(ii)) - 1;
    end

    % Clear assignments of data items to classes
    objpart.datacc{jj} = zeros(1,numdata);

    % Optionally, also clear category label to create a test group
    if clearcat
      objpart.category(jj) = 0;
    end
  end

  objpart.classqq = classqq;
  objpart.classnd = classnd;
  objpart.objnd   = objnd;

