
function [groupMean,groupCovar] = getreference(sceneobj, groups)
%getreference     Reference transformation corresponding to current assignments
%
%    [groupMean,groupCovar] = getreference(SO, groups)
%
%  Return current reference transformations computed by Gibbs sampler, where
%    groups     = groups/images to return transformations (by default, all)
%    groupMean  = matrix such that groupMean(:,j) is a dim*numobj vector 
%                 giving a reference position for each object in image j
%    groupCovar = matrix such that groupCovar(:,:,j) gives the posterior
%                 covariance for image j computed via EM approximation, or 
%                 all zeros if reference positions sampled (numEM=0)

if nargin < 2
  groups = [1:sceneobj.numgroup];
end

if sceneobj.useref
  groupMean  = sceneobj.groupmean(:,groups);
  groupCovar = sceneobj.groupcovar(:,:,groups);
else
  groupMean  = zeros(1,length(groups));
  groupCovar = zeros(1,1,length(groups));
end

