
%test_sceneobject    Synthetic data test of scene-object model

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% Clustering hyperparameters
numscene = 1;
numobj   = 3;
numclass = 8;
alpha    = 2;
gamma    = 1;

useApp = 1; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)
usePos = 1; % 1 to use Gaussian feature position model, 0 neglects positions
useRef = 1; % 1 to use reference position, 0 neglects reference position

% Gaussian hyperparameters and visualization
dataprior{1}.type    = 'gaussian';
dataprior{1}.emean   = zeros(2,1);
dataprior{1}.cmean   = 0.1;
dataprior{1}.ecovar  = eye(2);
dataprior{1}.ccovar  = 6;
plotstyle{1}.numbers = 1;

% Discrete hyperparameters and visualization
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = 30;
dataprior{2}.dirweight  = 0.1;
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 128;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim,1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];
plotstyle{3}.stdevscale = [0.5 2];

% Reference hyperparameters and visualization
if useRef
  refprior.emean  = zeros(2*numobj,1);
  refprior.cmean  = 0.1;
  refprior.ecovar = (2^2)*eye(2*numobj);
  refprior.ccovar = 5;

  refcovar = refprior.ecovar;
  refmean  = 6*rand(2*numobj,1) - 3;
  refparam = [refmean; refcovar(:)];
else
  refprior = [];
  refparam = [];
end

% Test corpus parameters
numgroup = 10;
numlabel = 5;
numdata  = 50;

% Learning parameters
numgibbs = 100;
numEM    = 0;
testiter = [50:5:numgibbs];

%%%%%%%%%%%%%%
%% Test script
%%%%%%%%%%%%%%

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end
if ~usePos
  dataprior = dataprior(2);
  plotstyle = plotstyle(2);
end

% Create figures to display inference results
for dd = 1:length(plotstyle)
  Htgt(dd) = figure('name', 'Ground truth');
  Hinf(dd) = figure('name', 'Inferred parts');
end

% Construct and visualize synthetic data set
SOtgt = sceneobject(numgroup,numscene,numobj,numclass,alpha,gamma,dataprior,refprior);
SOtgt = sampledata(SOtgt, [], 0, numdata, ones(1,numobj)/numobj, [], {}, refparam);
[tgtParam,tgtProb] = getclasses(SOtgt);

visualize(SOtgt, plotstyle, Htgt);
drawnow;

% Clear part labels from all groups, possibly keeping some object labels
SOinf = clearlabels(SOtgt,[1:numlabel],0);
SOinf = clearlabels(SOinf,[numlabel+1:numgroup],1);

% Sample assignments for labeled groups first (faster mixing)
fprintf(1,'Labeled groups: ');
SOinf = doGibbs(SOinf, 1, {plotstyle, Hinf}, [1:numlabel], [], numEM);

% Gibbs sampling for all groups
fprintf(1,'All groups: ');
[SOinf,testprob] = doGibbs(SOinf, numgibbs, {plotstyle, Hinf}, ...
  [1:numgroup], [numlabel+1:numgroup], numEM, testiter);
[infParam,infProb] = getclasses(SOinf);

