
function tdp = insertdata(tdp, newdata, group, label)
%insertdata     Add new data items to the specified group
%
%    TDP = insertdata(TDP, newdata, group, label)
%
%  Adds new data items to specified group, where
%    newdata  = matrix such that each column is a different data point,
%               or if multiple datatypes a cell array such that
%               newdata{dd} is the data matrix for datatype dd
%    group    = single group in which to add data (existing data
%               in that group is appended to, not replaced)
%    label    = optional group-specific label assigned to this set of 
%               data items (by default, label=0)

  if nargin < 3
    group = 1;
  end
  if nargin < 4
    label = 0;
  end

  % Check that newdata has proper number of types and is consistent
  if ~iscell(newdata)
    newdata = {newdata};
  end
  if length(newdata) ~= tdp.numtype
    error('Wrong number of types in newdata');
  end
  for dd = 2:tdp.numtype
    if size(newdata{dd},2) ~= size(newdata{1},2)
      error('Internal inconsistency in newdata');
    end
  end

  % Add newdata to specified group
  tdp.numdata(group) = tdp.numdata(group) + size(newdata{1},2);
  tdp.datatt{group}  = [tdp.datatt{group}, zeros(1, size(newdata{1},2))];
  for dd = 1:tdp.numtype
    tdp.datass{group}{dd}   = [tdp.datass{group}{dd}, newdata{dd}];
  end
  tdp.datall{group}  = [tdp.datall{group}, label*ones(1,size(newdata{1},2))];

