
function [partParam,partProb,objProb,objTran,objLabel] = getobjects(tdp)
%getobjects    Estimate posterior object/part parameters from current assignments
%
%    [partParam,partProb,objProb,objTran,objLabel] = getobjects(TDP)
%
%  Estimate parameters of parts associated with L current object categories, where
%    partParam = L-dim. cell array so that partParam{cc}{1} are part position
%                parameters, and partParam{cc}{2} are part appearance parameters
%    partProb  = L-dim. cell array so that partProb{cc} is vector giving
%                probability of each part of object cc
%    objProb   = vector giving probability of each object category
%    objTran   = columns are transformation parameters for each object category
%    objLabel  = vector giving user-specified label, if any, for each category

% data and transformation hyperparameters
numtype = 2;
dtype   = tdp.dtype;
dparam  = tdp.dparam;
ttype   = tdp.ttype;
tparam  = tdp.tparam;

% object state variables
numobj  = tdp.numobj;
objnt   = tdp.objnt;
objpp   = tdp.objpp;
objll   = tdp.objll;
numpart = tdp.numpart;

% part level state variables
partnd = tdp.partnd;
partqq = tdp.partqq;

% data distributions for each part
for cc = 1:numobj
  for dd = 1:numtype
    partParam{cc}{dd} = datastat_estparam(partqq{cc}{dd}, partnd{cc}, ...
      dtype(dd), dparam{dd});
  end
  partProb{cc} = partnd{cc}/sum(partnd{cc});
end

% probability of each object category
objProb = objnt/sum(objnt);

% distributions over object-specific transformations
objTran = transtat_estparam(objpp, objnt, ttype, tparam);

% labels associated with given categories
objLabel = max(objll,0);    % hide any negative labels

