
function [tableObj,tableProb,tableTran,tableAssign,partAssign]=gettables(tdp,groups)
%gettables    Transformed clusters, or tables, for specified groups
%
%    [tableObj,tableProb,tableTran,tableAssign,partAssign]=gettables(TDP,groups)
%
%  Estimate transformed objects, or tables, from current assignments, 
%  where all outputs are length(groups)-dim. cell arrays such that
%    tableObj{j}   = numtable-dim. vector giving object category for each table
%    tableProb{j}  = numtable-dim. vector giving P(table)
%    tableTran{j}  = columns are transformation parameters for each table
%    tableAssign{j}= index of table to which each data item in group assigned
%    partAssign{j} = index of part, of assigned object, for each data item

if nargin < 2
  groups = [1:tdp.numgroup];
end

% data structures to store results
tableObj    = cell(length(groups),1);
tableProb   = cell(length(groups),1);
tableTran   = cell(length(groups),1);
tableAssign = cell(length(groups),1);
partAssign  = cell(length(groups),1);

% compute transformed mixtures, or tables, for each specified group
for j=1:length(groups)
  jj = groups(j);

  datakk   = tdp.datakk{jj};
  datatt   = tdp.datatt{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  tableObj{j}    = tablecc;
  tableProb{j}   = tablend/sum(tablend);
  tableTran{j}   = tablerr;
  tableAssign{j} = abs(datatt); % hide manually specified table assignments
  partAssign{j}  = datakk;
end

