
%depth_test   Test depth likelihood code

clear all; close all;

% pixel coordinate determining projection ray
v  = [0.2 0]';
ss = [v; 0];

% 3D means of clusters
mu(:,1) = [0 0 2]';
mu(:,2) = [0 0 6]';
mu(:,3) = [0 0 10]';
rr = zeros(3,3);

% 3D covariances of clusters
Lam(:,:,1) = eye(3);
Lam(:,:,2) = diag([4 1 1]);
Lam(:,:,3) = eye(3);

% 3D covariances of transformations
Tran(:,:,1) = 25*eye(3);
Tran(:,:,2) = 25*eye(3);
Tran(:,:,3) = 25*eye(3);

% depth evaluation parameters
zgrid = linspace(1,15,100)';
numzz  = length(zgrid);
ztype  = 1;
zparam = [numzz; zgrid];

% convert 3D means/covariances to stored statistics
numcc = size(mu,2);
nd    = 1e8*ones(1,numcc);
nt    = 1e8*ones(1,numcc);
for cc = 1:numcc
  datasum  = mu(:,cc)*nd(cc);
  datachol = chol(Lam(:,:,cc)*nd(cc));
  logdet   = -0.5*log(det(Lam(:,:,cc)));
  qq(:,cc) = [datasum; datachol(:); logdet];

  tranchol = chol(Tran(:,:,cc)*nt(cc));
  logdet   = -0.5*log(det(Tran(:,:,cc)));
  pp(:,cc) = [tranchol(:); logdet];
end
dtype  = 2;
dparam = [3 0 0]';
ttype  = 1;
tparam = [3 0]';

% compute conditional likelihoods
clikC = depth_condlike(qq,nd,rr,ss,dtype,dparam,ztype,zparam);
clikM = depth_marglike(qq,nd,pp,nt,ss,dtype,dparam,ttype,tparam,ztype,zparam);

% compute likelihoods directly in 3D
testC = zeros(numzz,numcc);
testM = zeros(numzz,numcc);
for zz = 1:numzz
  u = [v*zgrid(zz); zgrid(zz)];
  for cc = 1:numcc
    delmu = u - mu(:,cc);

    testC(zz,cc) = -0.5*log(det(Lam(:,:,cc)));
    testC(zz,cc) = testC(zz,cc) - 0.5*delmu'*inv(Lam(:,:,cc))*delmu;

    LamT = Lam(:,:,cc) + Tran(:,:,cc);
    testM(zz,cc) = -0.5*log(det(LamT));
    testM(zz,cc) = testM(zz,cc) - 0.5*delmu'*inv(LamT)*delmu;
  end
end

plot(zgrid, sum(exp(clikC),2), '-b', zgrid, sum(exp(testC),2), '-r');
hold on;
plot(zgrid, sum(exp(clikM),2), '-b', zgrid, sum(exp(testM),2), '-r');

