
function tdp = setassign(tdp, group, labels)
%setassign    Set labels restricting object category assignments for some data
%
%    TDP = setassign(TDP, group, labels)
%
%  Sets labels restricting assignment of some features categories, where
%    group  = single group for which assignments set 
%    labels = object category for all features in that group, or single label 
%             for all features, where negative labels denote unallowed assignments, 
%             and 0 denotes unlabeled features (default)

  % default parameters
  if nargin < 2
    group = 1;
  end
  if nargin < 3
    labels = 0;
  end

  % check consistency of labels with amount of data
  numdata = tdp.numdata(group);
  if length(labels) == 1
    labels = labels*ones(1,numdata);
  elseif length(labels) ~= numdata
    error('Improper number of labels specified');
  end

  % check consistency of labels with current assignments
  datatt  = abs(tdp.datatt{group});
  tablecc = tdp.tablecc{group};
  objll   = tdp.objll;

  for ii = find(datatt > 0) 
    datall = objll(tablecc(datatt(ii)));

    if ((labels(ii) > 0) && (datall > 0) && (datall ~= labels(ii))) || ...
       (labels(ii) == -datall) 
      error('Labels inconsistent with current feature assignments');
    end
  end

  % set specified labels on data
  % will propagate to objll on next Gibbs sampling iteration
  tdp.datall{group} = labels;

