
%test_tdpscenepart    Synthetic data test of TDP object/part scene model

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

rand('state',0);
randn('state',0);

% clustering hyperparameters
gamma = 5;   % global object categories
alpha = 1;   % object instances per image
nu    = 1;   % parts per object (nu=0 constrains to single part)

% Gaussian position hyperparameters and visualization
dataprior{1}.type   = 'gaussian';
%dataprior{1}.type   = 'diagonalgaussian';
dataprior{1}.emean  = zeros(2,1);
dataprior{1}.cmean  = 0.1;
dataprior{1}.ecovar = 2*eye(2);
dataprior{1}.ccovar = 6;
plotstyle.range     = 20*[-1 1 -1 1];
plotstyle.colors    = {'r','g','b','c','m','y'};

% Discrete appearance hyperparameters 
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = 20;
dataprior{2}.dirweight  = 0.1;

% Transformation hyperparameters
tranprior.type   = 'zeromeangaussian';
%tranprior.type   = 'zeromeandiagonalgaussian';
tranprior.ccovar = 6;
tranprior.ecovar = 15*eye(2);

% Test corpus parameters
numgroup = 10;
numlabel = 2;
numdata  = 100;
maxobj   = 3;
maxtable = 3;
maxpart  = 3;

% Learning parameters
numgibbs   = 50;
numaux     = 5;
numconiter = 30;

%%%%%%%%%%%%%%
%% Test script
%%%%%%%%%%%%%%

% Create figures to display inference results
Htgt = figure('name', 'Ground truth');
Hinf = figure('name', 'Inferred objects');

% Construct and visualize synthetic data set
TDPtgt = tdpscenepart(numgroup, alpha, gamma, nu, dataprior, tranprior);
TDPtgt = sampledata(TDPtgt, [], numdata, maxobj, maxtable, maxpart);
[tgtPartParam,tgtPartProb,tgtObjProb] = getobjects(TDPtgt);

visualize(TDPtgt, [1:min(5,numgroup)], plotstyle, Htgt);
drawnow;

% Clear part assignments used to generate data
TDPinf = clearassign(TDPtgt,[1:numlabel],0);
TDPinf = clearassign(TDPinf,[numlabel+1:numgroup],1);

% Gibbs sampling learning
visparam = {[1:min(5,numgroup)], plotstyle, Hinf};
TDPinf = doGibbs(TDPinf, numgibbs, numaux, visparam, [1:numgroup], numconiter);
[infPartParam,infPartProb,infObjProb] = getobjects(TDPinf);

