
%computeSIFT    Precompute features for scene database using LabelMe toolbox

clear all;
close all;

% Descriptor parameters
scalingDescriptor = 1; % 1 = no scaling. %sqrt(2);
circularROI = 0;
scaling = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STREET SCENES
if true
  HOMEIMAGES  = '../../database/streetFixScale/Images'; 
  HOMEANNOTATIONS  = '../../database/streetFixScale/Annotations';

  objectlist = 'car,building,road,tree';
  depthorder = [1 2 3 4 0]; % average depth ordering (the closest object = 1)
  plotOrder = [0 3 2 1 4];
  imagesize = [256 342];

  featuresfilename = 'streetFixScale_2006_mser_ha_edges';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OFFICE SCENES
if false
  HOMEIMAGES  = '../../database/office/Images'; 
  HOMEANNOTATIONS  = '../../database/office/Annotations'; 
 
  objectlist = 'screen,mouse-pad,keyboard';
  depthorder = [1 2 3 0]; % average depth ordering (the closest object = 1)
  plotOrder = [0 4 3 2 1];
  imagesize = [256 256];
        
  featuresfilename = 'office_monocular_2006_mser_ha_edges';
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Create database
database = LMdatabase(HOMEANNOTATIONS);
database = LMquery(database, 'object.deleted', '0'); % remove deleted polygons

% Visualize the first 10 images (sanity check)
%disp('Show first 10 images...')
%LMdbshowscenes(database, HOMEIMAGES);

% count number of images in dataset
Nimages = length(database);

% Initialize output struct
clear imagedata
imagedata.scalingDescriptor = scalingDescriptor;
imagedata.circularROI = circularROI;
imagedata.objectlist = objectlist;

% loop for all images, load image and compute features
disp('Start Loop')
for n = 1:Nimages
    % 1) Load image
    filename = fullfile(database(n).annotation.folder, database(n).annotation.filename);
    img_left = imread(fullfile(HOMEIMAGES, filename));

    % Re-scale images
    %[database(n).annotation, img_left] = LMimscale(database(n).annotation, img_left, imagesize(1)/size(img_left,1), 'bilinear');

    % 2) Get SIFT descriptiors and locations Feat(1:5,:) = location and region cov. matrix, Feat(6:end,:) = SIFT descriptor
    [feat_ha_left, foo, type_left]  = features(uint8(img_left), ...
        'descriptor', 'sift', 'ROI', 'ha', ...
        'removeredund', 0, 'circularROI', circularROI, ...
        'removelargeregions', 1, 'scalingdescriptor', 1, ...
        'scaling', scaling);

    % 3) Assign a class to each feature
    % get segmentation mask for all the objects
    [mask, cc, maskpol, classpol] = LMobjectmask(database(n).annotation, [size(img_left,1) size(img_left,2)], objectlist);
    overlap = zeros(size(feat_ha_left,2),1);
    for c = 1:size(maskpol,3)
        growMask = imdilate(maskpol(:,:,c),ones(15,15));
        overlap(:,c) = overlapSegmentFeature(growMask, feat_ha_left);
    end

    % sort classes: we do this to assure that small objects are assigned features even if they overlap other objects.
    Jo = [];
    for c = depthorder
        Jo = [Jo; find(classpol==c)];
    end
    overlap = overlap(:,Jo);
    classpol = classpol(Jo);

    % assign classes and instances to each feature on the left image
    classes = zeros(1,size(overlap,1));
    instance = zeros(1,size(overlap,1));
    for c = 1:size(maskpol,3)
        j = find(overlap(:,c)>.3 & classes' == 0);
        if length(j)>0
            classes(j) = classpol(c);
            instance(j) = c;
        end
    end

    % reorder features in left image so that the plot order is the correct one:
    jo = [];
    for c = 1:length(plotOrder)
        jo = [jo find(classes==plotOrder(c))];
    end
    classes  = classes(jo);
    instance = instance(jo);
    feat_ha_left = feat_ha_left(:,jo);
    type_left = type_left(jo);
    overlap = overlap(jo,:);

    % get ROI properties: image location, region scale, ...
    [orientation, s, aspectRatio, x, y] = getROIproperties(feat_ha_left);

    % 5) Display detected features
    hfig = figure(1); clf
    subplot(231);
    LMplot(database(n).annotation, img_left); legend off
    title([n Nimages])
    title(database(n).annotation.filename)
    axis('on')

    figure(hfig)
    subplot(232);
    displayROI(img_left, feat_ha_left, instance+2);
    colormap(gray(256))

    figure(hfig)
    subplot(233);
    displayROI(img_left, feat_ha_left, type_left);
    colormap(gray(256))
    title([n Nimages])

    % 6) Concatenated with previous ones:
    imagedata.imgsize{n} = size(img_left);
    imagedata.descriptor_left{n} = uint8(feat_ha_left(6:end,:));
    imagedata.shapeROI_left{n}   = feat_ha_left(1:5,:);
    imagedata.typeROI_left{n}    = type_left;
    imagedata.classes{n}         = classes;
    imagedata.instance{n}        = instance; % polygon number
    imagedata.overlap{n}         = overlap'; % how much does the ROI overlap with the ground truth segmentation for each instance
    imagedata.files{n}           = fullfile(database(n).annotation.folder, database(n).annotation.filename);
    
    drawnow
end
imagedata.database = database;
imagedata.objectlist = objectlist;

% store
disp('Data saved in file:')
disp(featuresfilename)
save(featuresfilename, 'imagedata');

