
%encodeWords    Encodes features as visual words for sharing experiment 

% Load precomputed SIFT descriptors for all images
load imagesSharingIJCV_SIFT;
%classes  = [classes{:}]; % I made may life difficult before

% Load SIFT dictionary
load dictionaryVisualWordsSIFT;
useLocation = 0;  % set to 1 to also use region shape, increasing vocabulary size

% Names associated with the different classes (object categories)
className{1}  = 'Human Face';
className{2}  = 'Motorbike';
className{3}  = 'Background';
className{4}  = 'Bicycle';
className{5}  = 'Cannon';
className{6}  = 'Car';
className{7}  = 'Cat Face';
className{8}  = 'Cow Face';
className{9}  = 'Cougar Face';
className{10} = 'Dog Face';
className{11} = 'Horse Face';
className{12} = 'Leopard Face';
className{13} = 'Llama Face';
className{14} = 'Elephant Body';
className{15} = 'Horse Body';
className{16} = 'Leopard Body';
className{17} = 'Llama Body';
className{18} = 'Rhino Body';
className{19} = 'Wheelchair'; 

% Specify object and background classes, in desired order
objClass  = [7:18 2 4 5 19];
backClass = 3;

% Select groups corresponding to each specified class
ind        = find(ismember(classes, [objClass backClass]));
Descriptor = Descriptor(ind);  % 128-dim. SIFT descriptor
Location   = Location(ind);    % 5-dim. region location/shape 
file       = file(ind);        % image file name
classes    = classes(ind);     % object category of each image

numimage = length(file);

% Renumber object categories according to desired order
classesTmp = classes;
for jj = 1:length(objClass)
  ind = find(classesTmp == objClass(jj));
  classes(ind) = jj;
end

className = className(objClass);

% Assign background category to index 0
ind = find(ismember(classesTmp, backClass));
classes(ind) = 0;

% Compute dictionary word associated with every feature
words = cell(1,numimage);
for jj = 1:numimage
  disp(numimage-jj)

  if useLocation
    words{jj} = getWords(DICTvisualWords, Descriptor{jj}, Location{jj});
  else
    words{jj} = getWords(DICTvisualWords, Descriptor{jj});
  end
  words{jj} = words{jj}';
end

% Store computed features for later use
imagedata.dictionary = DICTvisualWords;
imagedata.numwords   = size(DICTvisualWords,1) * (1 + 2*useLocation);
imagedata.appearance = words;
imagedata.position   = Location;
imagedata.classes    = classes;
imagedata.files      = file;
imagedata.numclasses = length(objClass);
imagedata.classnames = className;
imagedata.descriptor = Descriptor;

