function words = getWords(centers, descriptor, locations)
% words = getWords(centers, descriptor, locations)
%
% The function can be used in two ways:
% 1) Returns the cluster index for each feature
% words = getWords(centers, descriptor);
%
% 2) Enhances dictionary with aspect ratio and orientation information
% words = getWords(centers, descriptor, locations)

descriptor = double(descriptor);
NumberWords = size(centers,1);

% find closest center to each descriptor:
[words, d] = matchDescriptors(descriptor, centers');

% Enhance dictionary with aspect ratio and orientation information
if nargin == 3
    [orientation, scale, aspectRatio] = getROIproperties(locations);

    % Enhances the dictionary by making the size of the dictionary three
    % times bigger by taking into account the shape of the region. 
    % First looks at the aspect ratio. One group is for regions with
    % aspect ratio is < 2. Then for aspectRatio>2 there are two groups
    % according to the orientation of the region (>45 or <45 degrees)
    wordOrient = abs(cos(orientation)) < sqrt(2)/2;
    wordScale  = aspectRatio > 2;
    ndx = wordScale.*(1+wordOrient);
    words = words + ndx*NumberWords;
end




