
function [aROC, alarmRate, detectRate] = perfDetect(objllik, backllik, truecat)
%perfDetect     Compute statistics summarizing detection performance

% if several samples, approximate average likelihood by max
objllik  = max(objllik, [], 3);
backllik = max(backllik, [], 3);

% divide log-likelihood by constant to avoid underflow (ROC unchanged)
objllik  = objllik/100;
backllik = backllik/100;

% compute detection performance for each category versus background
numobj = size(objllik,1);
for oo = 1:numobj
  % select examples of background and current object category
  testind = find((truecat==0) | (truecat==oo));

  % compute posterior probabilities for these examples
  postprob = [objllik(oo,testind); backllik(1,testind)];
  postprob = exp(postprob - repmat(max(postprob,[],1),2,1));
  postprob = postprob ./ repmat(sum(postprob,1),2,1);

  % compute ROC curve (alarmRate is same for all categories)
  [aROC(oo), alarmRate, detectRate(oo,:)] = ...
    areaROC(postprob(1,:), (truecat(testind)==oo));
end

