
function [aROC, alarmRate, detectRate, confMatrix] = perfRecog(objllik, truecat)
%perfRecog    Compute statistics summarizing recognition performance

% if several samples, approximate average likelihood by max
objllik  = max(objllik, [], 3);

% divide log-likelihood by constant to avoid underflow (ROC unchanged)
objllik  = objllik/100;

% discard any test images containing background
testind = find(truecat > 0);
objllik = objllik(:,testind);
truecat = truecat(testind);

% compute posterior probabilities of each object category, for all examples
numobj   = size(objllik,1);
postprob = exp(objllik - repmat(max(objllik,[],1),numobj,1));
postprob = postprob ./ repmat(sum(postprob,1),numobj,1);

% compute ROC curves (alarmRate is same for all categories)
for oo = 1:numobj
  [aROC(oo), alarmRate, detectRate(oo,:)] = areaROC(postprob(oo,:), (truecat==oo));
end

% confusion matrix for choice among all categories
if nargout > 3
  [bestprob,bestcat] = max(postprob, [], 1);
  confMatrix = confusionMatrix(truecat, bestcat);
end

