
% Visualization of LDA scene model: learned contextual relationships

useStreet = 0;

dim = 2;
if useStreet
  objstyle = {'r','m','b','g'};
  objcond  = 1;
  objplot  = [2:4];
  objpart  = [1:4];

  threshpart = 0.05;
  scenerange = [0 342 0 256];

  reffile = 'streetPosLDA_test10';
else
  objstyle = {'r','b','g','k'};
  objcond  = 1;
  objplot  = [2 3];
  objpart  = [1:4];

  threshpart = 0.05;
  scenerange = [0 256 0 256];

  reffile = 'officePosLDA_test10';
end

% load simulation results
result = load(reffile, 'SO');
[sceneProb,refMean,refCovar] = getscenes(result.SO);
[classParam,classProb]       = getclasses(result.SO);

% compute conditional mean and covariances
numplot   = length(objplot);
postMean  = zeros(dim,numplot);
postCovar = zeros(dim,dim,numplot);

fixMean = refMean(dim*(objcond-1)+1:dim*objcond);
for ii = 1:numplot
  % x --> object to be plotted (unobserved)
  % y --> reference object (observation)
  Mx  = refMean(dim*(objplot(ii)-1)+1:dim*objplot(ii));
  My  = refMean(dim*(objcond-1)+1:dim*objcond);

  Pxx = refCovar(dim*(objplot(ii)-1)+1:dim*objplot(ii), ...
    dim*(objplot(ii)-1)+1:dim*objplot(ii));
  Pxy = refCovar(dim*(objplot(ii)-1)+1:dim*objplot(ii), ...
    dim*(objcond-1)+1:dim*objcond);
  Pyy = refCovar(dim*(objcond-1)+1:dim*objcond, ...
    dim*(objcond-1)+1:dim*objcond);

  postMean(:,ii)    = Mx + Pxy*inv(Pyy)*(fixMean - My);
  postCovar(:,:,ii) = Pxx - Pxy*inv(Pyy)*Pxy';

  % shift mean by weighted average of part means
  postMean(:,ii) = postMean(:,ii) + ...
    classParam{1}(1:dim,:) * classProb(objplot(ii),:)';
end

% shift reference by weighted average of part means
fixMean = fixMean + classParam{1}(1:dim,:) * classProb(objcond,:)';

% plot posterior mean and covariance
Hfig = figure;
Hax  = subplot(1,1,1,'Parent',Hfig);
hold(Hax,'on');
plot(Hax, fixMean(1), fixMean(2), strcat('o',objstyle{objcond}), ...
  'MarkerFaceColor', objstyle{objcond}, 'MarkerSize', 12);
for ii = 1:numplot
  plotGaussMix(postMean(:,ii), postCovar(:,:,ii), 1, ...
    objstyle{objplot(ii)}, Hax, {'o','MarkerSize',12}, ...
    {strcat('-',objstyle{objplot(ii)}), 'LineWidth', 3});
end
axis(Hax,'equal'); axis(Hax,'ij'); axis(Hax,scenerange);
set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);

% plot parts for all objects at their mean locations
Hfig = figure;
Hax  = subplot(1,1,1,'Parent',Hfig);
hold(Hax,'on');
for oo = objpart
  cc = find(classProb(oo,:) > threshpart);
  mu = classParam{1}(1:dim,cc) + refMean(dim*(oo-1)+1:dim*oo, ones(1,length(cc)));
  sigma = reshape(classParam{1}(dim+1:end,cc), dim, dim, length(cc));

  plotGaussMix(mu, sigma, classProb(oo,cc), objstyle{oo}, Hax, ...
    {'o', 'MarkerSize', 10});
end
axis(Hax,'equal'); axis(Hax,'ij'); axis(Hax,scenerange);
set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);

