
% Visualization for LDA scene model: feature segmentation performance

useStreet = 1;
useRef    = 1;
useMax    = 0;  % true to take max-likelihood sample, false to average

if useStreet
  newlabel = 0;  % exclude unlabeled regions from test
  objstyle = {'-r','-m','-b','-g'};

  if useRef
    filename = 'streetPosLDA_test10';
  else
    filename = 'streetAppLDA_test';
  end
else
  newlabel = 4;  % assume unlabeled regions are background
  objstyle = {'-r','-b','-g','-k'};

  if useRef
    filename = 'officePosLDA_test10';
  else
    filename = 'officeAppLDA_test';
  end
end

% plot parameters
axisROC  = [0 1 0 1];

% compute log-likelihood of each test image interpretation
load(filename);
numtrialTest = size(objprob,1);

% create figures to display results
HfigD = figure('name', 'Detection'); 

% extract test image probabilities and true labels
testprob = zeros(numobj,0); testlabel = [];
for kk = 1:length(testgroup)
  objlabel = imagedata.classes{testgroup(kk)};
  objlabel(find(objlabel==0)) = newlabel;

  testind   = find(objlabel > 0);
  testlabel = [testlabel objlabel(testind)];

  if useMax
    [trialval,trialind] = max(sceneprob(:,kk));
    newprob = sum(objprob{trialind,kk}(testind,:,:),3)';
  else
    newprob = sum(objprob{1,kk}(testind,:,:),3)';
    for ii = 2:numtrialTest
      newprob = newprob + sum(objprob{ii,kk}(testind,:,:),3)';
    end
    newprob = newprob/numtrialTest;
  end
  testprob = [testprob newprob];
end

% detection performance for each object category
for oo = 1:numobj
  [aROC(oo),alarmRate,detectRate(oo,:)] = areaROC(testprob(oo,:), (testlabel==oo));
end

% plot ROC curves
Hax = subplot(1,1,1,'Parent',HfigD); hold(Hax,'on');
for oo = 1:numobj
  plot(Hax, alarmRate, detectRate(oo,:), objstyle{oo}, 'linewidth', 3);
end
axis(Hax,'square'); axis(Hax, axisROC); grid(Hax,'on');

