
% Visualization of image segmentations for contextual LDA scene model

useStreet = 0;
useRef    = 0;

datapath   = '../database';

if useStreet
  imagepath = fullfile(datapath, 'streetFixScale/Images');
  plotorder = [2 3 4 1];
  plotcolor = [1 0 0; 1 0 1; 0 0 1; 0 1 0];
  plotstyle  = {'r', 'm', 'b', 'g'};

  % don't display overly small test images
  showImages = [6:11 14:16 19 24 27];
  numcolumn  = 6;

  if useRef
    filename = 'streetPosLDA_test10';
    objprior  = ones(4,1);
  else
    filename = 'streetAppLDA_test';
    objprior = [1.5 1 2 3]';
  end
else
  imagepath = fullfile(datapath, 'office/Images');
  plotorder  = [4 1 3 2];
  plotcolor  = [1 0 0; 0 0 1; 0 1 0; 0.5 0.5 0.5];
  plotstyle  = {'r', 'b', 'g', 'k'};

  % don't display overly small test images
  showImages = [1 3 5 6 7 10 11 12 13 14 15 21 22 28];
  numcolumn  = 7;

  if useRef
    filename = 'officePosLDA_test10';
    objprior  = ones(4,1);
  else
    filename = 'officeAppLDA_test';
    objprior = [2 3 3 1]';
  end
end

% load simulation results
load(filename);
docs = testgroup(1:max(showImages));
numtrialTest = size(objprob,1);

% plot parameters
plotcolorparts = [1 1 0; 1 .5 0; 0 1 .5; 0 1 1; .5 .5 .5; 1 0 .5; 1 .5 0; ...
  1 0 1; 1 .25 .5; 0 .5 1; 1 .8 .5; .8 1 .8; .7 1 .6];
Nx = numcolumn; Ny = 4; % will show 7x8 images per figure
sp = 0; spl=0;

% plot segmented features for each image
figure
for kk = showImages
    % load and display original image
    rawname  = fullfile(imagepath, imagedata.files{docs(kk)});
    rawimage = imread(rawname);

    % 1) show image alone
    if spl >= Nx; figure; spl = 0; end
    sp = spl;
    axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny]); sp = sp+Nx; 
    imshow(rawimage);

    % 2) show segmentation averaging across reference samples
    
    % determine mean probability for each feature assignment
    objprobMean = sum(objprob{1,kk},3)';
    for ii = 2:numtrialTest
      objprobMean = objprobMean + sum(objprob{ii,kk},3)';
    end
    objprobMean = objprobMean/numtrialTest;

    numfeat = size(imagedata.shapeROI_left{docs(kk)},2);
    [maxval,objlabel] = max(objprobMean .* repmat(objprior,1,numfeat));

    % change order in which features displayed
    newind = [];
    for oo = 1:numobj
        newind = [newind find(objlabel==plotorder(oo))];
    end

    axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny]); sp = sp+Nx; % create axis
    displayROI(rawimage, imagedata.shapeROI_left{docs(kk)}(:,newind), ...
        objlabel(newind), plotcolor, min(maxval(newind),1)); title('')

    % select most likely reference sample
    [foo, ii] = max(sceneprob(:,kk));
    [maxval,objlabel] = max(sum(objprob{ii,kk},3)' .* repmat(objprior,1,numfeat));

    if useRef
        sampdata = load(sprintf('%s%02d',filename(1:end-2),ii),'SO');
        
        % 3) Plot part locations relative to object reference frames for each image
        axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny]); sp = sp+Nx; 
        imshow(rawimage); hold on

        %classParam{1} % part locations
        %classProb % each row = p(part|obj)
        [classParam,classProb] = getclasses(sampdata.SO);

        % reference positions for each object/image:
        objref = getreference(sampdata.SO, docs(kk));
        for oo = plotorder
            % only show parts assigned to objects that are present in the image
            if ismember(oo, objlabel) 
                c = find(classProb(oo,:)>.05); % find parts assigned to object 'oo'
                mu = classParam{1}(1:2,c) + repmat(objref(2*oo-1:2*oo), [1 length(c)]);
                th = reshape(classParam{1}(3:6,c), [2 2 length(c)]);

                plotGaussMix(mu, th, classProb(oo,c), plotstyle{oo}, gca)
            end
        end

        % 4) Show Parts
        objclassprob = objprob{ii,kk};

        selectClass = 1; % screen/office, car/street
        jo = find(objlabel==selectClass);
        objclassprob = objclassprob(jo, selectClass, :);
        [foo, classlabel] = max(objclassprob, [], 3);

        uniquelabel  = unique(classlabel);
        classrelabel = zeros(size(classlabel));
        for jj = 1:length(uniquelabel)
          classrelabel(find(classlabel==uniquelabel(jj))) = jj;
        end

        axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny]); sp = sp+Nx; 
        displayROI(rawimage, imagedata.shapeROI_left{docs(kk)}(:,jo), ...
            classrelabel, plotcolorparts); title('')
    end
    spl = spl+1;
    %input(sprintf('Test image %d...',kk));
end

