
% Visualization of IJCV05 TDP scenes: image segmentations
function scenepartTDP_visseg(TDP, groups, testProb, imagedata, imagepath, ...
  plotorder, plotcolor, plotstyle, plotdetail, plottable, Hfig)

if nargin < 9
  plotdetail = 1;
end
if nargin < 10
  plottable = [];
end
if nargin < 11
  Hfig = figure;
else
  clf(Hfig);
end

% process arguments
numcat = length(plotorder); % number of user-specified categories
dim    = 2;

% transformed clusters for displayed images
[tableObj,tableProb,tableTran,tableAssign,partAssign] = gettables(TDP, groups);
[partParam,partProb,objProb,objTran,objLabel] = getobjects(TDP);
numobj   = length(objProb);   % number of global object clusters
objLabel = max(objLabel,0);   % ignore negative labels

% extract mean & covariance of parts for each object
muPart  = cell(1,numobj);
sigPart = cell(1,numobj);

for cc = 1:numobj
  numpart(cc) = length(partProb{cc});
  muPart{cc}  = partParam{cc}{1}(1:dim,:);

  if size(partParam{cc}{1},1) > 2*dim
    sigPart{cc} = reshape(partParam{cc}{1}(dim+1:end,:),dim,dim,numpart(cc));
  else
    sigPart{cc} = zeros(dim,dim,numpart(cc));
    for kk = 1:numpart(cc)
      sigPart{cc}(:,:,kk) = diag(partParam{cc}{1}(dim+1:end,kk));
    end
  end
end

% plot parameters
plotcolortable = [1 1 0; 1 .5 0; 0 1 .5; 0 1 1; .5 .5 .5; 1 0 .5; 1 .5 0; ...
  1 0 1; 1 .25 .5; 0 .5 1; 1 .8 .5; .8 1 .8; .7 1 .6];
Nx = 6; Ny = 4; % will show 7x8 images per figure
sp = 0; spl=0;

% plot segmented features for each image
for j = 1:length(groups)
  jj = groups(j);

  % look for instances that are assigned to at least 10 features 
  numfeat = length(tableAssign{j});
  plotind = find(tableProb{j} > 10/numfeat);

% 1) show image alone
  rawname  = fullfile(imagepath, imagedata.files{jj});
  rawimage = imread(rawname);

  if spl >= Nx; Hfig = figure; spl = 0; end
  sp = spl;
  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+Nx; 
  image(rawimage,'parent',Hax); axis(Hax,'image'); axis(Hax,'off');

% 2) show segmentation based on current assignments
  if isempty(testProb)
    featObj    = zeros(1,numfeat);
    assignind  = find(tableAssign{j} > 0);
    featObj(assignind) = objLabel(tableObj{j}(tableAssign{j}(assignind)));
    featWeight = ones(1,numfeat);
  else
    numTest  = prod(size(testProb));
    featMean = testProb{1}{jj};
    for ii = 2:numTest
      featMean = featMean + testProb{ii}{jj};
    end
    featMean = featMean / numTest;

    [featWeight,featObj] = max(featMean);

    % estassign puts probabilities of unassigned visual categories last
    featObj(featObj == size(featMean,1)) = 0;
  end

  % change order in which features displayed
  newind = [];
  for oo = 1:numcat
    newind = [newind find(featObj == plotorder(oo))];
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+Nx; 
  displayROI(rawimage, imagedata.shapeROI_left{jj}(:,newind), ...
    featObj(newind)+1, plotcolor, featWeight(newind), Hax); title(Hax,'');
  
% 3) show transformed tables
  muT = []; sigmaT = []; weightsT = []; colorsT = {};
  for tt = plotind
    cc = tableObj{j}(tt);

    if (isempty(plottable) || ismember(objLabel(cc),plottable)) 
      muT      = cat(2, muT, muPart{cc}+tableTran{j}(:,tt*ones(1,numpart(cc))));
      sigmaT   = cat(3, sigmaT, sigPart{cc});
      weightsT = cat(2, weightsT, tableProb{j}(tt)*partProb{cc});
      colorsT  = cat(2, colorsT, repmat(plotstyle(objLabel(cc)+1),1,numpart(cc)));
    end
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+Nx; 
  image(rawimage,'parent',Hax); axis(Hax,'image'); axis(Hax,'off');
  plotGaussMix(muT, sigmaT, weightsT, colorsT, Hax);

% 4) show instances corresponding to specified detail class
  selectTable = plotind(find(objLabel(tableObj{j}(plotind)) == plotdetail));

  dataind  = []; datalabel = [];
  partbase = 0; 
  for t = 1:length(selectTable)
    tt = selectTable(t);

    newdata   = find(tableAssign{j} == tt);
    dataind   = [dataind, newdata];
    datalabel = [datalabel, partbase + partAssign{j}(newdata)];
    partbase  = partbase + numpart(tableObj{j}(tt));
  end

  Hax = axes('position', [mod(sp,Nx)/Nx (Ny-fix(sp/Nx)-1)/Ny 1/Nx 1/Ny], ...
    'parent', Hfig); sp = sp+Nx; 
  displayROI(rawimage, imagedata.shapeROI_left{jj}(:,dataind), ...
    datalabel, plotcolortable, [], Hax); title(Hax,'');
  spl = spl+1;
end

