
% Visualize parts learned by HDP model

% load raw data 
load sharing_ijcv16;    

% load result file
load posS6b_test30 trainhist usePos;
DP = trainhist{end}.DPinfO{end};
imagefolder = '../database/imagesSharingIJCV/';

% extract classes/tables from learned model
[classParam,classProb,classTran] = getclasses(DP);
[tableClass,tableProb,tableTran,tableAssign] = gettables(DP);
numobj   = length(tableProb);
numclass = length(classProb);

% plot parameters
numimage  = 6;        % number of images of each category 
classind  = [43 63 108 113 114 125 133];  % indices of parts to visualize 
featcolor = [1 1 0];  % RGB color of feature ellipses
% different color for each table
tablecolor = [1 1 0; 0 1 1; 1 .5 0; 0 1 .5; .5 .5 .5; 1 0 .5; 1 .5 0; 1 0 1; 0 0 0];
tablecolor = repmat(tablecolor,4,1);

% load images used for later display, and compute part assignments
objimage    = cell(numobj,numimage);
objassign   = cell(numobj,numimage);
objposition = cell(numobj,numimage);

fprintf(1,'Loading images for all categories: ');
for oo = 1:numobj
  objind = find(imagedata.classes == oo);
  for ii = 1:numimage
    objimage{oo,ii}    = imread(fullfile(imagefolder,imagedata.files{objind(ii)}));
    objposition{oo,ii} = imagedata.position{objind(ii)};

    testdata{1} = objposition{oo,ii}(1:2,:) + 1;
    testdata{2} = imagedata.appearance{objind(ii)};
    objassign{oo,ii} = estassign(DP, oo, testdata);
  end
  fprintf(1,'.');
end
fprintf(1,'\n');

% sort parts by H(obj|part)
objProb = zeros(numobj,numclass);
for oo = 1:numobj
  for tt = 1:length(tableProb{oo})
    cc = tableClass{oo}(tt);
    objProb(oo,cc) = objProb(oo,cc) + tableProb{oo}(tt);
  end
end
objProb = (objProb+eps) ./ repmat(sum(objProb+eps,2),1,numclass);

if numobj > 1
  entPart = objProb ./ repmat(sum(objProb,1),numobj,1);
  entPart = -sum(entPart.*log(entPart),1);
  [entSort,entIndex] = sort(entPart);
else
  [entSort,entIndex] = sort(-objProb);
end

% add dummy index to allow visualization of uninstantiated class
entIndex = [entIndex, numclass+1];

% plot position distribution for each part
dim = 2;
classMean  = classParam{1}(1:dim,:);
classCovar = reshape(classParam{1}(dim+1:end,:),dim,dim,numclass);

% realign mean values to account for empirical mean of current tables
muTables  = zeros(dim,numclass);
if usePos > 1
  tranCovar = reshape(classTran,dim,dim,numclass);
  classnt   = zeros(1,numclass);

  for jj = 1:numobj
    tablecc  = tableClass{jj};
    tablerr  = tableTran{jj};

    for tt = 1:length(tablecc)
      muTables(:,tablecc(tt)) = muTables(:,tablecc(tt)) + tablerr(:,tt);
      classnt(tablecc(tt))    = classnt(tablecc(tt)) + 1;
    end
  end
  muTables = muTables ./ classnt(ones(dim,1),:);
end

HfigG = figure(numclass+2);
set(HfigG, 'name', 'Part Position Densities');
for j = 1:numclass
  jj  = entIndex(j);
  Hax = subplot(ceil(numclass/8),8,j,'Parent',HfigG);
  plotGaussMix(classMean(:,jj)+muTables(:,jj), classCovar(:,:,jj), ...
    1, 'b', Hax, {}, {'-k', 'LineWidth', 2});

  if usePos > 1
    hold(Hax,'on');
    plotGaussMix(classMean(:,jj)+muTables(:,jj), tranCovar(:,:,jj), ...
      1, 'b', Hax, {}, {'--k', 'LineWidth', 1});
  end

  axis(Hax,'square'); axis(Hax,'ij'); axis(Hax,[0 128 0 128]);
  set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
  title(Hax, sprintf('%d',j));
end
input('Part position densities...');

% plot regions associated with each chosen part in turn
Dy = 1/numobj;
Dx = 1/(numimage+1); 
%Dx = Dy;

for j = classind
  jj = entIndex(j);

  HfigP = figure(j);
  set(HfigP, 'name', sprintf('Part %d (ordered by entropy)',j));

  % for each object category, and each image in that category
  for oo = 1:numobj
    tableNames = find(tableClass{oo}==jj);
    numtable   = length(tableNames);

    for ii = 1:numimage
      % find all features most likely to be generated by this part
      [partval,partind] = max(objassign{oo,ii}(:,1:end-1), [], 2);

      % TDP returns table assignments, HDP class assignments
      if usePos > 1
        plotind = find(tableClass{oo}(partind) == jj);

        % renumber table assignments
        tableassign = partind(plotind);
        newassign   = zeros(size(tableassign));
        for tt = 1:numtable
          newassign(find(tableassign==tableNames(tt))) = tt;
        end
      else
        plotind = find(partind == jj);
      end

      % plot those features
      axes('position', [(ii-1)*Dx (numobj-oo)*Dy Dx Dy]);
      if usePos > 1
        displayROI(objimage{oo,ii}, objposition{oo,ii}(:,plotind), ...
          newassign, tablecolor);
      else
        displayROI(objimage{oo,ii}, objposition{oo,ii}(:,plotind), ...
          [], featcolor);
      end
      title(''); colormap(gray(256));
    end

    % plot tables corresponding to each object category
    if usePos > 1
      ii = numimage+1;
      Hax = axes('position', [(ii-1)*Dx (numobj-oo)*Dy Dx Dy]);
      if numtable > 0
        tableMean  = repmat(classMean(:,jj),1,numtable)+tableTran{oo}(:,tableNames);
        tableCovar = repmat(classCovar(:,:,jj),[1 1 numtable]);
        plotGaussMix(tableMean, tableCovar, tableProb{oo}(tableNames), ...
          'b', Hax, {'o','MarkerSize',2}, {'-k','LineWidth',1});
      end
      axis(Hax,'square'); axis(Hax,'ij'); axis(Hax,[0 128 0 128]);
      set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
    end
  end
  
  input(sprintf('Part %d...',j));
end

% compute distances between distributions over parts
D = zeros(numobj);
for ii = 1:numobj
  for jj = 1:numobj
    D(ii,jj) = sum(objProb(ii,:).*log(objProb(ii,:)./objProb(jj,:))) + ...
               sum(objProb(jj,:).*log(objProb(jj,:)./objProb(ii,:)));
  end
end

if numobj == 1
  return;
end

% dendrogram
Z = linkage(squareform(D), 'single');
HfigD = figure(numclass+3);
set(HfigD, 'name', 'Category Distance Dendrogram');
Hax = subplot(1,1,1,'Parent',HfigD);
[Hline,ind,perm] = dendrogram(Z,'colorthreshold','default','orientation','right');
set(Hax, 'Yticklabel', imagedata.classnames(perm));
set(Hax, 'FontSize', 8);

% MDS embedding
coord = mdscale(D,2,'Criterion','metricstress');
coord = coord - min(min(coord));
coord = 0.85*coord/max(max(coord));

HfigM = figure(numclass+4);
set(HfigM, 'name', 'Category Distance Embedding');
for oo = 1:numobj
  axes('position', [coord(oo,1) coord(oo,2) .15 .15])
  image(objimage{oo,1});
  title(imagedata.classnames{oo});
  hold on;
  plot(64,64,'or','markerfacecolor','r');
  axis('off'); axis('equal'); axis('tight');
  colormap(gray(256));
end

