
% TDP/HDP/DP (transformed/shared/unshared) models for object categories
% Visualization of detection/recognition performance:

% load raw data
load sharing_ijcv16;    

% load result file(s): 
result{1}{1,1} = 'IJCV05-09-29/posN_test12';
result{1}{1,2} = 'IJCV05-09-29/posN_test30';
result{2}{1,1} = 'PHD06-03-30/posS6a_test12';
result{2}{1,2} = 'PHD06-03-30/posS6b_test30';
result{3}{1,1} = 'IJCV05-09-29/appN_test30';
result{4}{1,1} = 'PHD06-03-30/appS6_test30';
descrip = {'DP','HDP','DP Appearance','HDP Appearance'};
style   = {'-g','-r','--g','--r'};

% plot parameters
axisROC  = [0 0.5 0 1];
axisArea = [0.5 1 0.5 1];
plotArea = 1;   % plot individual category ROC, or ROC area scatter plot
countEnt = 0;   % true to use part entropy to count, false to just count
numpointsROC = 320;

% extract needed variables
numtrial  = length(result);
params    = load(result{1}{1}, 'numobj', 'testindex');
numobj    = params.numobj;
%testindex = params.testindex;

numtrainO = [];
for jj = 1:size(result{1},2)
  params = load(result{1}{1,jj}, 'numtrainO');
  numtrainO = [numtrainO, params.numtrainO];
end

% avoid excessive memory usage, but collect statistics on number of parts
trainhist = cell(length(numtrainO),numtrial);
meanpart  = zeros(length(numtrainO),numtrial);
varpart   = zeros(length(numtrainO),numtrial);
for ii = 1:numtrial
  resultind = 1;

  [numseed,numproc] = size(result{ii});

  for jj = 1:numproc
    for ss = 1:numseed
      traintemp = load(result{ii}{ss,jj}, 'trainhist');

      if ss == 1
        numresult = length(traintemp.trainhist);
        totalsamp = zeros(numresult,1);
      else
        if length(traintemp.trainhist) ~= numresult
          error('Different random seeds must use same training cases');
        end
      end

      for kk = 1:numresult
        if isempty(traintemp.trainhist{kk})
          break;
        end
  
        numsamp = length(traintemp.trainhist{kk}.DPinfO);
        aa      = resultind + kk - 1;
        for bb = 1:numsamp
          [classParam,classProb] = getclasses(traintemp.trainhist{kk}.DPinfO{bb});
          if countEnt
            numclass = exp(-sum(classProb.*log(classProb)));
          else
            numclass = length(classProb);
          end

          meanpart(aa,ii) = meanpart(aa,ii) + numclass;
          varpart(aa,ii)  = varpart(aa,ii) + numclass^2;
          totalsamp(kk)   = totalsamp(kk) + 1;
        end

        if isfield(traintemp.trainhist{kk}, 'DPinfB')
          traintemp.trainhist{kk}.DPinfB = {};
        end
        traintemp.trainhist{kk}.DPinfO = {};
      end

      if ss == 1
        trainhist(resultind:resultind+numresult-1,ii) = traintemp.trainhist;
      else
        for kk = 1:numresult
          if (trainhist{resultind+kk-1,ii}.traingroup ~= ...
              traintemp.trainhist{kk}.traingroup)
            error('Training groups for different seeds inconsistent');
          end
          trainhist{resultind+kk-1,ii}.probO = cat(3, ...
            trainhist{resultind+kk-1,ii}.probO, traintemp.trainhist{kk}.probO);
        end
      end

      clear traintemp;
    end

    crntind = [resultind:resultind+numresult-1];
    meanpart(crntind,ii) = meanpart(crntind,ii)./totalsamp;
    varpart(crntind,ii)  = varpart(crntind,ii)./totalsamp - meanpart(crntind,ii).^2;

    resultind = resultind + numresult;
  end
end

% Average performance for all numbers of training examples
detectArea = zeros(numobj,length(numtrainO),numtrial);
recogArea  = zeros(numobj,length(numtrainO),numtrial);
detectROC  = zeros(numpointsROC,length(numtrainO),numtrial);
recogROC   = zeros(numpointsROC,length(numtrainO),numtrial);

% Create figures to display results
HfigD = figure('name', 'Detection'); 
HfigR = figure('name', 'Recognition'); 

for kk = 1:length(numtrainO)
  % check whether probabilities calculated for this training set size
  if ~isfield(trainhist{kk,1},'probO');
    break;
  end

  % extract test groups, and true category labels
  testgroup = trainhist{kk,1}.testgroup;
  truecat   = imagedata.classes(testgroup);

  % detection performance
  for ii = 1:numtrial
    objllik  = zeros(numobj, length(testgroup), 0);
    backllik = zeros(1, length(testgroup), 0);
    for jj = 1:size(trainhist{kk,ii}.probO,3);
      probO = trainhist{kk,ii}.probO(:,:,jj);
      if min(min(probO)) < 0
        objllik  = cat(3, objllik, probO(1:numobj,:));
        backllik = cat(3, backllik, probO(numobj+1,:));
      end
    end

    [areaO,alarmO,detectO] = perfDetect(objllik, backllik, truecat);
    detectArea(:,kk,ii) = areaO';
    detectROC(:,kk,ii)  = mean(detectO)';
  end

  % plot detection
  if numtrial > 1
    Hax = subplot(2,1,1,'Parent',HfigD);
    cla(Hax); hold(Hax,'on');

    plot(Hax, detectArea(:,kk,1), detectArea(:,kk,2), ...
      'ob', 'markersize', 6, 'markerfacecolor', 'b');
    plot(Hax, [0 1], [0 1], '--k', 'linewidth', 2);
    axis(Hax,'square'); axis(Hax, axisArea);
    xlabel(Hax,descrip{1});
    ylabel(Hax,descrip{2});
    title(Hax,'Scatter plot of ROC Areas');

    Hax = subplot(2,1,2,'Parent',HfigD);
  else
    Hax = subplot(1,1,1,'Parent',HfigD);
  end
  cla(Hax); hold(Hax,'on');
  for ii = 1:numtrial
    plot(Hax, alarmO, detectROC(:,kk,ii), style{ii}, 'linewidth', 3);
  end
  axis(Hax,'square'); axis(Hax, axisROC);
  title(Hax,'Mean ROC curves');
  legend(Hax,descrip{:},'Location','SouthEast');

  % recognition performance
  for ii = 1:numtrial
    objllik  = zeros(numobj, length(testgroup), 0);
    for jj = 1:size(trainhist{kk,ii}.probO,3);
      probO = trainhist{kk,ii}.probO(:,:,jj);
      if min(min(probO)) < 0
        objllik = cat(3, objllik, probO(1:numobj,:));
      end
    end

    [areaO,alarmO,detectO,confO] = perfRecog(objllik, truecat);
    recogArea(:,kk,ii) = areaO';
    recogROC(:,kk,ii)  = mean(detectO)';
  end

  % plot recognition
  if numtrial > 1
    Hax = subplot(2,1,1,'Parent',HfigR);
    cla(Hax); hold(Hax,'on');

    plot(Hax, recogArea(:,kk,1), recogArea(:,kk,2), ...
      'ob', 'markersize', 6, 'markerfacecolor', 'b');
    plot(Hax, [0 1], [0 1], '--k', 'linewidth', 2);
    axis(Hax,'square'); axis(Hax, axisArea);
    xlabel(Hax,descrip{1});
    ylabel(Hax,descrip{2});
    title(Hax,'Scatter plot of ROC Areas');

    Hax = subplot(2,1,2,'Parent',HfigR);
  else
    Hax = subplot(1,1,1,'Parent',HfigR);
  end
  cla(Hax); hold(Hax,'on');
  for ii = 1:numtrial
    plot(Hax, alarmO, recogROC(:,kk,ii), style{ii}, 'linewidth', 3);
  end
  axis(Hax,'square'); axis(Hax, axisROC);
  title(Hax,'Mean ROC curves');
  legend(Hax,descrip{:},'Location','SouthEast');

  input(sprintf('%d training examples...',numtrainO(kk)));
end

% plot performance summary versus training examples
HfigT = figure('name', 'Training Examples'); 
Hax = subplot(2,1,1,'Parent',HfigT); hold(Hax,'on');
for ii = 1:numtrial
  plot(Hax, numtrainO, mean(detectArea(:,:,ii)), style{ii}, 'linewidth', 3);
end
Hax = subplot(2,1,2,'Parent',HfigT); hold(Hax,'on');
for ii = 1:numtrial
  plot(Hax, numtrainO, mean(recogArea(:,:,ii)), style{ii}, 'linewidth', 3);
end

% plot summary statistics of posterior over number of parts
HfigP = figure('name', 'Number of Parts');
Hax = subplot(1,1,1,'Parent',HfigP); hold(Hax,'on');
for ii = 1:numtrial
  plot(Hax, numtrainO, meanpart(:,ii), style{ii}, 'linewidth', 3);
  plot(Hax, numtrainO, meanpart(:,ii)+sqrt(varpart(:,ii)), style{ii}, 'linewidth', 2);
  plot(Hax, numtrainO, meanpart(:,ii)-sqrt(varpart(:,ii)), style{ii}, 'linewidth', 2);
end

for ii = 1:numtrial
  disp(['Detection ' descrip{ii}]); disp(mean(detectArea(:,:,ii)));
end
for ii = 1:numtrial
  disp(['Recognition ' descrip{ii}]); disp(mean(recogArea(:,:,ii)));
end

