
function C = confusionMatrix(truecat, testcat)
%confusionMatrix     Confusion matrix for given categorizations
%
%    C = confusionMatrix(truecat, testcat)
%
%  Compute confusion matrix, where
%    truecat = vector of true categories for test cases (positive integers)
%    testcat = vector of categories chosen by test algorithm
%          C = confusion matrix with a row for each true category, such that
%              C(i,j) = Pr[testcat=j | truecat=i]

n = max(truecat);
m = max(testcat);
C = zeros(n,m);

for ii = 1:n
  for jj = 1:m
    C(ii,jj) = sum((truecat==ii).*(testcat==jj));
  end
end

C = C ./ repmat(sum(C,2),1,m);

