function s = invwishart_rnd(Lambda, nu, n)
%invwishart_rnd     Sample from inverse-Wishart distribution
%
%    ddraw = invwishart_rnd(Lambda, nu, N)
%
%  Draws N independent samples from an inverse-Wishart distribution, where
%    Lambda = dxd symmetric, positive definite scale matrix
%    nu     = degrees of freedom (assumed integral)
%    ddraw  = dx(d*N) matrix of dxd matrix samples

if (nargin < 3)
  n=1;
end

d = size(Lambda,1);
s = zeros(d,n*d);

sqrtScale = inv(chol(Lambda));

for (i = 1:n)
  samp = sqrtScale*randn(d,nu);
  for (j = 1:nu)
    s(:,d*(i-1)+1:d*i) = s(:,d*(i-1)+1:d*i) + samp(:,j)*samp(:,j)';
  end
  s(:,d*(i-1)+1:d*i) = inv(s(:,d*(i-1)+1:d*i));
end  

